package MKDoc::Apache::Auth::MySQL;
use Apache::Constants qw/:common/;
use 5.6.1;
use strict;
use warnings;
use DBI;

our $VERSION = '0.3';

use constant DEFAULT_HOSTNAME => 'localhost';
use constant DEFAULT_USERNAME => 'root';
use constant DEFAULT_PASSWORD => undef;

sub handler
{
    my $r = shift;
    
    # checks that the user has sent a login / password pair
    my ($res, $sent_pw) = $r->get_basic_auth_pw;
    return $res if $res != OK;
    
    my $user = $r->connection->user;
    unless ($user and $sent_pw)
    {
        $r->note_basic_auth_failure;
        $r->log_reason ("Both username and passwords must be provided, ", $r->filename);
        return AUTH_REQUIRED;
    }
    
    # attempts to connect to the MySQL database
    my $db_username = $r->dir_config ('mysql_user')
	|| $r->dir_config ('mysql_username')
	|| DEFAULT_USERNAME;
    
    my $db_password = $r->dir_config ('mysql_pwd')
	|| $r->dir_config ('mysql_passwd')
	|| $r->dir_config ('mysql_password')
	|| DEFAULT_PASSWORD;
    
    my $dsn = _compute_dsn ($r);
    my $dbh = undef;
    eval { $dbh = DBI->connect ($dsn, $db_username, $db_password, { RaiseError => 1, AutoCommit => 1 }) };
    if (defined $@ and $@)
    {
	$r->log_error ("MKDoc::Apache::Auth::MySQL: Cannot connect: $@");
	return SERVER_ERROR;
    }
    
    # fetches the user's password
    my $sth = $dbh->prepare_cached ('SELECT Password FROM Editor WHERE Login=?');
    $sth->execute ($user);
    my $array_ref = $sth->fetchrow_arrayref;

    $dbh->disconnect;

    # if $res is not defined, then the user does not exist
    return AUTH_REQUIRED unless (defined $array_ref); 
    # otherwise, let's compare the passwords
    return AUTH_REQUIRED unless ($array_ref->[0] eq $sent_pw);
    
    # the user is authenticated
    return OK;
}


sub _compute_dsn
{
    my $r = shift;
    my $database = $r->dir_config ('mysql_database');
    my $port     = $r->dir_config ('mysql_port') || undef;
    my $hostname = $r->dir_config ('mysql_host')
	|| $r->dir_config ('mysql_hostname')
	|| DEFAULT_HOSTNAME;
    
    my $dsn = "DBI:mysql:database=$database";
    $dsn   .= ":host=$hostname" if (defined $hostname and $hostname);
    $dsn   .= ":port=$port" if (defined $port and $port);
    return $dsn;
}



1;
__END__

=head1 NAME

MKDoc::Apache::Auth::MySQL - MKDoc MySQL Authentication module
