#!/bin/sh
# 
# This is a script to build a tree of symlinked mkdoc templates, it
# avoids maintaining unnecessary duplicates of template files.
# 
# Usage (notice ABSOLUTE PATHS):
# 
#   rm -rf /mkdoc/sites/mysite/templates
#   sh create-linked-tree.sh /mkdoc/sites/default/templates /mkdoc/sites/mysite/templates
# 
# To create a real file that can be edited, break a symlink like this:
# 
#   cp index.html index.html.new
#   rm index.html
#   mv index.html.new index.html
# 
# Run this script as many times as you like, it won't clobber real
# template files, but will fill-in the gaps when stuff is missing.

# trim any trailing '/'s from the command-line parameters

SOURCE=`echo $1 | sed s:/$::`

DESTINATION=`echo $2 | sed s:/$::`

# directories shouldn't be linked, create them instead.

find $SOURCE -type d | grep -v CVS |
    while read line
    do \
        path=$DESTINATION/`echo $line | sed s:$SOURCE::`
        mkdir --verbose $path
    done

# files need to be sym-linked

find $SOURCE -type f | grep -v CVS |
    while read line
    do \
        path=$DESTINATION/`echo $line | sed s:$SOURCE::`
        ln -v -s $line $path
    done

