package HTML_Page::Dmoz;
use base qw /HTML_Page/;
use strict;
use warnings;
use URI;


# Dmoz title look like: 'Open Directory - Arts: Animation: Anime: Titles'
# We want it to be 'Titles' instead
sub title
{
    my $self = shift;
    my $title = $self->SUPER::title;
    $title =~ s/.*?\-//;
    $title =~ s/.*\://;
    $title =~ s/^(\s|\r|\n)+//gsm;
    $title =~ s/(\s|\r|\n)+$//gsm;
    return $title;
}


# for Dmoz, we want to return all the external links as MKDoc components
sub body
{
    my $self = shift;
    $self->load;
    $self->is_ok or return;
    
    my $self_URI = new URI ($self->uri);
    my %res  = ();
    my $data = $self->{content};
    my $tree = new HTML::TreeBuilder;
    $tree->parse ($data);
    $tree->elementify;
    foreach (@{$tree->extract_links ('a')})
    {
	my ($link, $element, $attr, $tag) = @$_;
	my $base_URI = new URI ($self->uri);
	my $link_URI = new URI ($link);
	$link_URI = $link_URI->abs ($base_URI);
	$link = $link_URI->canonical;
	next unless (defined $link_URI->scheme);
	next unless ($link_URI->scheme =~ /^https?/);
	next if ($self_URI->host eq $link_URI->host);
	my ($title) = $element->content_list();
	$title = "$title";
	my $href = "$link";
	$res{$link} = {
	    href  => $href,
	    title => $title || $href,
	    desc  => $element->attr ('title') || $href,
	};
    }
    
    my $count = 0;
    return join '', map {
	$count++;
	"<block id=\"flo_block_$count\" type=\"link\">\n" .
	"<a href=\"$_->{href}\" title=\"$_->{desc}\">$_->{title}</a>\n" .
	"</block>\n"
    } sort { $a->{title} cmp $b->{title} } values %res;
}


sub children
{
    my $self = shift;
    my $uri = new URI ($self->uri);
    my $uri_path = $uri->path;
    my @uri_slashcount = $uri_path =~ /(\/)/g;
    my $uri_slashcount = scalar @uri_slashcount;
    
    my @result = ();
    foreach my $link ($self->internal_links)
    {
	my $link_uri = new URI ($link);
	my $link_uri_path = $link_uri->path;
	my @link_uri_slashcount = $link_uri_path =~ /(\/)/g;
	my $link_uri_slashcount = scalar @link_uri_slashcount;
	
	push @result, $link
 	    if ( $link_uri_slashcount == $uri_slashcount + 1 and
		 $link_uri_path =~ /^\Q$uri_path\E/ );
    }
    
    return sort @result;
}


1;


__END__
