package HTML_Page::Links_SQL;
use base qw /HTML_Page::Dmoz/;
use strict;
use warnings;
use URI;
use Unicode::String qw /latin1/;

use vars qw /$floBlockCount/;


# for Dmoz, we want to return all the external links as MKDoc components
sub body
{
    my $self = shift;
    $self->load;
    $self->is_ok or return;
    local $floBlockCount = 0;
    my $data = $self->{content};
    my $tree = new HTML::TreeBuilder;
    $tree->parse ($data);
    $tree->p_strict (1);
    $tree->elementify;
    
    my $res = join "\n", (
	$self->body_header ($tree),
	$self->body_links  ($tree),
	$self->body_footer ($tree),
	$self->body_menu   ($tree)
       );
    
    return $res;
}


sub body_header
{
    my $self = shift;
    my @nodes = shift;
    while (@nodes)
    {
	my $current = shift (@nodes);
	if (ref $current)
	{
	    my $class_attr = $current->attr ('class');
	    if (defined $class_attr and $class_attr eq 'links-header')
	    {
		my $u_res = latin1 ($current->as_HTML);
		my $res = $u_res->utf8;
		$floBlockCount++;
		return join "\n", (
		    "<block id=\"flo_block_$floBlockCount\" type=\"html\">\n",
		    "$res\n",
		    "</block>"
		   );
	    }
	    else
	    {
		push @nodes, $current->content_list;
	    }
	}
    }
    
    $floBlockCount++;
    return join '', (
	"<block id=\"flo_block_$floBlockCount\" type=\"html\">\n",
	'',
	"</block>"
       );
}


sub body_footer
{
    my $self = shift;
    my @nodes = shift;
    while (@nodes)
    {
	my $current = shift (@nodes);
	if (ref $current)
	{
	    my $class_attr = $current->attr ('class');
	    if (defined $class_attr and $class_attr eq 'links-footer')
	    {
		my $u_res = latin1 ($current->as_HTML);
		my $res = $u_res->utf8;
		$floBlockCount++;
		return join "\n", (
		    "<block id=\"flo_block_$floBlockCount\" type=\"html\">\n",
		    $res,
		    "</block>"
		   );
	    }
	    else
	    {
		push @nodes, $current->content_list;
	    }
	}
    }
    
    $floBlockCount++;
    return join '', (
	"<block id=\"flo_block_$floBlockCount\" type=\"html\">\n",
	'',
	"</block>"
       );
}


sub body_menu
{
    my $self = shift;
    my @nodes = shift;
    while (@nodes)
    {
	my $current = shift (@nodes);
	if (ref $current)
	{
	    my $class_attr = $current->attr ('class');
	    if (defined $class_attr and $class_attr eq 'links-menu')
	    {
		my $u_res = latin1 ($current->as_HTML);
		my $res = $u_res->utf8;
		$floBlockCount++;
		return join "\n", (
		    "<block id=\"flo_block_$floBlockCount\" type=\"html\">\n",
		    "$res\n",
		    "</block>"
		   );
	    }
	    else
	    {
		push @nodes, $current->content_list;
	    }
	}
    }
    
    $floBlockCount++;
    return join '', (
	"<block id=\"flo_block_$floBlockCount\" type=\"html\">\n",
	'',
	"</block>"
       );
}


sub body_links
{    
    my $self = shift;
    my $tree = shift;
    my %res  = ();
    foreach (@{$tree->extract_links ('a')})
    {
	my ($link, $element, $attr, $tag) = @$_;
	next unless ($link =~ /\/jump\.cgi/);
	my ($title) = $element->content_list();
	my $href = "$link";
	my $URI = URI->new ($href);
	my %form = $URI->query_form;
	delete $form{t};
	delete $form{d};
	$URI->query_form (%form);
	$href = $URI->as_string;
	
	$res{$href} = {
	    href  => $href,
	    title => $title || $href,
	    desc  => $element->attr ('title') || '',
	};
	$res{$href}->{desc}  =~ s/\n//gsm;
	$res{$href}->{title} =~ s/\n//gsm;
	$res{$href}->{href}  =~ s/\n//gsm;
	$res{$href}->{desc}  =~ s/\r//gsm;
	$res{$href}->{title} =~ s/\r//gsm;
	$res{$href}->{href}  =~ s/\r//gsm;
    }
    
    my $links = join "\n", map {
	$floBlockCount++;
	"<block id=\"flo_block_$floBlockCount\" type=\"link\">\n" .
	"<a href=\"$_->{href}\" title=\"$_->{desc}\">$_->{title}</a>\n" .
	"</block>"
    } sort { $a->{title} cmp $b->{title} } values %res;
    my $u_links = latin1 ($links);
    return $u_links->utf8;
}


# Dmoz title look like: 'Open Directory - Arts: Animation: Anime: Titles'
# We want it to be 'Titles' instead
sub title
{
    my $self = shift;
    my $title = $self->SUPER::title;
    $title =~ s/.*?\-//;
    $title =~ s/.*\://;
    $title =~ s/.*\///;
    $title =~ s/^(\s|\r|\n)+//gsm;
    $title =~ s/(\s|\r|\n)+$//gsm;
    return $title;
}


sub full_path
{
   my $self = shift;
   my $uri = new URI ($self->uri);
   my %form = $uri->query_form;
   my $path = $form{g} || '';
   $path = "/$path" unless ($path =~ /^\//);
   $path =~ s/\/index.html$/\//;
   return $path;
}


sub internal_links
{
    my $self = shift;
    my @links = $self->links;
    
    return map {
	my $uri = $_;
	my $uri_obj = URI->new ($_);
	my %hash = $uri_obj->query_form;
	( defined $hash{'g'} && $_ =~ /\/page\.cgi/ ) ? $_ : ()
    } @links;
}


sub external_links
{
    my $self = shift;
    my @links = $self->links;
    return map { $_ =~ /\/rate\.cgi/ ? $_ : () } @links;
}


sub children
{
    my $self = shift;
    my $uri = new URI ($self->uri);
    my %form = $uri->query_form;
    my $uri_path = $self->full_path;
    my @uri_slashcount = $uri_path =~ /(\/)/g;
    my $uri_slashcount = scalar @uri_slashcount;
    
    my @result = ();
    foreach my $link ($self->internal_links)
    {
	my $link_uri = new URI ($link);
	my %form = $link_uri->query_form;
	my $link_uri_path = $form{g} || next;
	$link_uri_path = "/$link_uri_path" unless ($link_uri_path =~ /^\//);
	my @link_uri_slashcount = $link_uri_path =~ /(\/)/g;
	my $link_uri_slashcount = scalar @link_uri_slashcount;
	
	push @result, $link
	    if ( ( $link_uri_slashcount == $uri_slashcount + 1 and
		   $link_uri_path =~ /^\Q$uri_path\E/ ) or
		   $link_uri_path eq '/cool' );
    }
    
    return sort @result;
}


1;


__END__
